/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikavaluearray.h ported from GValueArray
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_BASE_H_INSIDE__) && !defined (PIKA_BASE_COMPILATION)
#error "Only <libpikabase/pikabase.h> can be included directly."
#endif

#ifndef __PIKA_VALUE_ARRAY_H__
#define __PIKA_VALUE_ARRAY_H__

G_BEGIN_DECLS

/**
 * PIKA_TYPE_VALUE_ARRAY:
 *
 * The type ID of the "PikaValueArray" type which is a boxed type,
 * used to pass around pointers to PikaValueArrays.
 *
 * Since: 2.10
 */
#define PIKA_TYPE_VALUE_ARRAY (pika_value_array_get_type ())


GType            pika_value_array_get_type (void) G_GNUC_CONST;

PikaValueArray * pika_value_array_new      (gint                  n_prealloced);
PikaValueArray * pika_value_array_new_from_types
                                           (gchar               **error_msg,
                                            GType                 first_type,
                                            ...);
PikaValueArray * pika_value_array_new_from_types_valist
                                           (gchar               **error_msg,
                                            GType                 first_type,
                                            va_list               va_args);
PikaValueArray * pika_value_array_new_from_values
                                           (const GValue *values,
                                            gint          n_values);

PikaValueArray * pika_value_array_copy     (const PikaValueArray *value_array);

PikaValueArray * pika_value_array_ref      (PikaValueArray       *value_array);
void             pika_value_array_unref    (PikaValueArray       *value_array);

gint             pika_value_array_length   (const PikaValueArray *value_array);

GValue         * pika_value_array_index    (const PikaValueArray *value_array,
                                            gint                  index);

PikaValueArray * pika_value_array_prepend  (PikaValueArray       *value_array,
                                            const GValue         *value);
PikaValueArray * pika_value_array_append   (PikaValueArray       *value_array,
                                            const GValue         *value);
PikaValueArray * pika_value_array_insert   (PikaValueArray       *value_array,
                                            gint                  index,
                                            const GValue         *value);

PikaValueArray * pika_value_array_remove   (PikaValueArray       *value_array,
                                            gint                  index);
void             pika_value_array_truncate (PikaValueArray       *value_array,
                                            gint                  n_values);


/*
 * PIKA_TYPE_PARAM_VALUE_ARRAY
 */

#define PIKA_TYPE_PARAM_VALUE_ARRAY           (pika_param_value_array_get_type ())
#define PIKA_IS_PARAM_SPEC_VALUE_ARRAY(pspec) (G_TYPE_CHECK_INSTANCE_TYPE ((pspec), PIKA_TYPE_PARAM_VALUE_ARRAY))
#define PIKA_PARAM_SPEC_VALUE_ARRAY(pspec)    (G_TYPE_CHECK_INSTANCE_CAST ((pspec), PIKA_TYPE_PARAM_VALUE_ARRAY, PikaParamSpecValueArray))

typedef struct _PikaParamSpecValueArray PikaParamSpecValueArray;

/**
 * PikaParamSpecValueArray:
 * @parent_instance:  private #GParamSpec portion
 * @element_spec:     the #GParamSpec of the array elements
 * @fixed_n_elements: default length of the array
 *
 * A #GParamSpec derived structure that contains the meta data for
 * value array properties.
 **/
struct _PikaParamSpecValueArray
{
  GParamSpec  parent_instance;
  GParamSpec *element_spec;
  gint        fixed_n_elements;
};

GType        pika_param_value_array_get_type (void) G_GNUC_CONST;

GParamSpec * pika_param_spec_value_array     (const gchar    *name,
                                              const gchar    *nick,
                                              const gchar    *blurb,
                                              GParamSpec     *element_spec,
                                              GParamFlags     flags);


G_END_DECLS

#endif /* __PIKA_VALUE_ARRAY_H__ */
